# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Run feature depending on provided args"""
import subprocess
import sys
from pathlib import Path
from typing import List, Optional

from scargo.config_utils import prepare_config
from scargo.logger import get_logger

logger = get_logger()


def scargo_run(bin_path: Optional[Path], profile: str, params: List[str]) -> None:
    """
    Run command from CLI

    :param str bin_path: path to bin file
    :param Path profile: build profile name
    :param str params: params for bin file
    :return: None
    """
    logger.info('Running "%s" build', profile)

    config = prepare_config()
    if "x86" not in config.project.target.family:
        logger.info(
            "Run project on x86 architecture is not implemented for %s yet.",
            config.project.target.family,
        )
        sys.exit(1)

    if bin_path:
        bin_file_name = bin_path.name
        bin_file_path = bin_path.parent
        try:
            subprocess.check_call([f"./{bin_file_name}"] + params, cwd=bin_file_path)
        except subprocess.CalledProcessError:
            logger.error("bin file not found")
    else:
        project_profile_path = config.project_root / "build" / profile
        bin_dir = project_profile_path / "bin"
        if bin_dir.is_dir():
            first_bin = next(bin_dir.iterdir())
            # Run project
            try:
                subprocess.check_call([f"./{first_bin.name}"] + params, cwd=bin_dir)
            except subprocess.CalledProcessError:
                logger.error("Unable to run bin file")
        else:
            logger.error("Bin file not found")



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from scargo.config import Config
from scargo.file_generators.base_gen import create_file_from_template


def generate_readme(config: Config) -> None:
    """Generate dirs and files"""
    create_file_from_template(
        "README.md.j2",
        "README.md",
        overwrite=False,
        template_params={"project": config.project},
        config=config,
    )



import sys
from pathlib import Path
from typing import Optional

import tomlkit

from scargo import __version__
from scargo.config import Config, ConfigError, parse_config
from scargo.docker_utils import run_scargo_again_in_docker
from scargo.global_values import SCARGO_LOCK_FILE
from scargo.logger import get_logger
from scargo.path_utils import get_config_file_path

logger = get_logger()


def get_scargo_config_or_exit(
    config_file_path: Optional[Path] = None,
) -> Config:
    """
    :param config_file_path
    :return: project configuration as dict
    """
    if config_file_path is None:
        config_file_path = get_config_file_path(SCARGO_LOCK_FILE)
    if config_file_path is None or not config_file_path.exists():
        logger.error("File `%s` does not exist.", SCARGO_LOCK_FILE)
        logger.info("Did you run `scargo update`?")
        sys.exit(1)

    try:
        config = parse_config(config_file_path)
    except ConfigError as e:
        logger.error(e.args[0])
        sys.exit(1)

    if compiler_warning := config.project.get_compiler_warning():
        logger.warning(compiler_warning)
    return config


def prepare_config(run_in_docker: bool = True) -> Config:
    """
    Prepare configuration file

    :return: project configuration
    """
    config = get_scargo_config_or_exit()
    check_scargo_version(config)
    if run_in_docker:
        run_scargo_again_in_docker(config.project, config.project_root)
    return config


def check_scargo_version(config: Config) -> None:
    """
    Check scargo version

    :param Config config: project configuration
    :return: None
    """
    version_lock = config.scargo.version
    if __version__ != version_lock:
        logger.warning("Warning: scargo package is different then in lock file")
        logger.info("Run scargo update")


def add_version_to_scargo_lock(scargo_lock: Path) -> None:
    """
    :return: project configuration as dict
    """
    with open(scargo_lock, encoding="utf-8") as scargo_lock_file:
        config = tomlkit.load(scargo_lock_file)

    config.setdefault("scargo", tomlkit.table())["version"] = __version__
    with open(scargo_lock, "w", encoding="utf-8") as scargo_lock_file:
        tomlkit.dump(config, scargo_lock_file)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from pathlib import Path
from typing import Any, Dict, Union

from jinja2 import Environment, FileSystemLoader

from scargo.config import Config
from scargo.global_values import SCARGO_PKG_PATH
from scargo.logger import get_logger

logger = get_logger()

_TEMPLATE_ROOT = Path(SCARGO_PKG_PATH, "file_generators", "templates")
_JINJA_ENV = Environment(
    loader=FileSystemLoader(_TEMPLATE_ROOT),
    trim_blocks=True,
    lstrip_blocks=True,
)


def create_file_from_template(
    template_path: str,
    output_path: Union[Path, str],
    template_params: Dict[str, Any],
    config: Config,
    overwrite: bool = True,
) -> None:
    """Creates file using jinja template on output path, creates dirs if necessary"""
    project_path = config.project_root
    output_path = Path(project_path, output_path)
    if (
        _is_file_excluded(output_path, project_path, config)
        or output_path.exists()
        and not overwrite
    ):
        return
    write_template(output_path, template_path, template_params)


def write_template(
    output_path: Path, template_path: str, template_params: Dict[str, Any]
) -> None:
    output_path.parent.mkdir(parents=True, exist_ok=True)
    content = _JINJA_ENV.get_template(template_path).render(template_params)
    output_path.write_text(content, encoding="utf-8")

    logger.info("Generated %s", output_path)


def _is_file_excluded(output_path: Path, project_path: Path, config: Config) -> bool:
    exclude_list = config.scargo.update_exclude
    return str(output_path.relative_to(project_path)) in exclude_list



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Update project"""
import shutil
import subprocess
import sys
from pathlib import Path

from scargo.commands.docker import get_docker_compose_command, scargo_docker_build
from scargo.config_utils import add_version_to_scargo_lock, get_scargo_config_or_exit
from scargo.file_generators.cicd_gen import generate_cicd
from scargo.file_generators.cmake_gen import generate_cmake
from scargo.file_generators.conan_gen import generate_conanfile, generate_conanprofile
from scargo.file_generators.docker_gen import generate_docker_compose
from scargo.file_generators.env_gen import generate_env
from scargo.file_generators.readme_gen import generate_readme
from scargo.file_generators.tests_gen import generate_tests
from scargo.global_values import SCARGO_DOCKER_ENV, SCARGO_LOCK_FILE, SCARGO_PKG_PATH
from scargo.logger import get_logger

logger = get_logger()


def copy_file_if_not_exists(project_path: Path) -> None:
    """
    Copy file from scargo pkg

    :return: None
    """
    files_to_copy = Path(SCARGO_PKG_PATH, "templates").glob("*")
    for file in files_to_copy:
        if not Path(project_path, file.name).exists():
            shutil.copy2(file, project_path)


def scargo_update(config_file_path: Path) -> None:
    """
    Update project

    :param config_file_path: path to .toml configuration file (e.g. scargo.toml)
    :return: None
    """
    project_path = config_file_path.parent
    docker_path = Path(project_path, ".devcontainer")
    config = get_scargo_config_or_exit(config_file_path)
    if not config.project:
        logger.error("File `%s`: Section `project` not found.", config_file_path)
        sys.exit(1)

    if not config.project.name:
        logger.error(
            "File `{config_file_path}`: `name` not found under `project` section."
        )
        sys.exit(1)

    # Copy templates project files to repo directory
    copy_file_if_not_exists(project_path)

    # Copy config file and create lock file.
    lock_path = project_path / SCARGO_LOCK_FILE
    shutil.copyfile(config_file_path, lock_path)
    ###########################################################################
    add_version_to_scargo_lock(lock_path)
    project_config = config.project
    target = project_config.target

    # Copy docker env files to repo directory
    generate_docker_compose(docker_path, config)
    generate_env(docker_path, config)

    generate_cmake(config)
    generate_conanfile(config)
    generate_conanprofile(config)

    if target.family == "esp32":
        Path(config.source_dir_path, "fs").mkdir(parents=True, exist_ok=True)
        with open(Path(project_path, "version.txt"), "w", encoding="utf-8") as out:
            out.write(project_config.version)
        with open(Path(project_path, "partitions.csv"), "w", encoding="utf-8") as out:
            out.write(
                "# ESP-IDF Partition Table\n# Name,   Type, SubType, Offset,  Size, Flags\n"
            )
            partitions = config.get_esp32_config().partitions
            for line in partitions:
                out.write(line + "\n")
            out.write("\n")

    generate_cicd(config=config)
    generate_tests(config)
    generate_readme(config)

    # do not rebuild dockers in the docker
    if (
        target.family == "stm32"
        and not Path(config.project_root, "third-party/stm32-cmake").is_dir()
    ):
        subprocess.run("conan source .", shell=True, cwd=project_path, check=True)

    if project_config.build_env == SCARGO_DOCKER_ENV:
        if not Path("/.dockerenv").exists():
            if not pull_docker_image(docker_path):
                scargo_docker_build([], config.project_root)
        else:
            logger.warning("Cannot run docker inside docker")


def pull_docker_image(docker_path: Path) -> bool:
    logger.info("Pulling the image from docker registry...")
    try:
        cmd = get_docker_compose_command()
        cmd.extend(["pull"])
        result = subprocess.run(
            cmd,
            cwd=docker_path,
            stderr=subprocess.PIPE,
            check=True,
        )
    except subprocess.CalledProcessError as e:
        logger.warning(e.stderr.decode())
    else:
        # happens for the default tag, like "myproject-dev:1.0"
        if (
            "Some service image(s) must be built from source"
            not in result.stderr.decode()
        ):
            logger.info("Docker image pulled successfully")
            return True
    return False



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Generate toml for scargo project"""
from pathlib import Path
from typing import Any

from scargo.file_generators.base_gen import write_template


def generate_toml(output_file_path: Path, **values: Any) -> None:
    """_summary_

    Args:
        output_file_path (String): path to the output .env file
        **values (Dict): dict contains all necessary values for toml
    """
    write_template(output_file_path, "scargo.toml.j2", template_params=values)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Global values for scargo"""
import pkgutil
from pathlib import Path

DESCRIPTION = "C/C++ package and software development life cycle manager based on RUST cargo idea."

SCARGO_PKG_PATH = (
    Path(pkgutil.get_loader("scargo").path).parent  # type: ignore[union-attr]
    if pkgutil.get_loader("scargo").path  # type: ignore[union-attr]
    else Path(__file__).parent
)
SCARGO_DEFAULT_BUILD_ENV = "docker"
SCARGO_DOCKER_ENV = "docker"

SCARGO_LOCK_FILE = "scargo.lock"
SCARGO_DEFAULT_CONFIG_FILE = "scargo.toml"
ENV_DEFAULT_NAME = ".env"



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

from scargo.config import Config
from scargo.file_generators.base_gen import create_file_from_template


def generate_conanfile(config: Config) -> None:
    create_file_from_template(
        "conan/conanfile.py.j2",
        "conanfile.py",
        template_params={"config": config},
        config=config,
    )
    create_file_from_template(
        "conan/conanfiletest.j2",
        "tests/conanfile.py",
        template_params={"config": config},
        config=config,
    )


def generate_conanprofile(config: Config) -> None:
    profiles = list(config.profiles.keys())
    for profile in profiles:
        create_file_from_template(
            "conan/profile.j2",
            f".conan/profiles/{config.project.target.family}_{profile}",
            template_params={"config": config, "profile": profile},
            config=config,
        )



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

import subprocess
import sys
from pathlib import Path
from time import sleep
from typing import List, Optional

from scargo.config import Config
from scargo.config_utils import prepare_config
from scargo.docker_utils import run_scargo_again_in_docker
from scargo.logger import get_logger
from scargo.path_utils import find_program_path

logger = get_logger()


class _ScargoDebug:
    SUPPORTED_TARGETS = ["x86", "stm32", "esp32"]

    def __init__(self, config: Config, bin_path: Optional[Path]):
        self._target = config.project.target
        self._project_root = config.project_root

        if self._target.family not in self.SUPPORTED_TARGETS:
            logger.error("Debugging currently not supported for %s", self._target)
            logger.info(
                "Scargo currently supports debug for %s", self.SUPPORTED_TARGETS
            )
            sys.exit(1)

        self._bin_path = bin_path or self._get_bin_path(config.project.name.lower())
        if not self._bin_path.exists():
            logger.error("Binary %s does not exist", self._bin_path)
            logger.info("Did you run scargo build --profile Debug?")
            sys.exit(1)

        if self._target.family == "stm32":
            stm32_config = config.get_stm32_config()
            self._chip = stm32_config.chip
            if not self._chip:
                logger.error("Chip label not defined in toml.")
                logger.info("Define chip under stm32 section and run scargo update.")
                sys.exit(1)

    def run_debugger(self) -> None:
        if self._target.family == "x86":
            self._debug_x86()
        elif self._target.family == "stm32":
            self._debug_stm32()
        elif self._target.family == "esp32":
            self._debug_esp32()

    def _debug_x86(self) -> None:
        subprocess.run(["gdb", self._bin_path], check=False)

    def _debug_embedded(self, openocd_args: List[str], gdb_bin: str) -> None:
        openocd_path = find_program_path("openocd")
        if not openocd_path:
            logger.error("Could not find openocd.")
            sys.exit(1)
        openocd_call = [openocd_path] + openocd_args

        openocd = subprocess.Popen(openocd_call)  # pylint: disable=consider-using-with
        # Wait for openocd to start
        sleep(1)
        try:
            subprocess.run(
                [
                    gdb_bin,
                    self._bin_path,
                    "--eval-command=target extended-remote localhost:3333",
                ],
                check=True,
            )
        finally:
            openocd.terminate()

    def _debug_stm32(self) -> None:
        chip_script = f"target/{self._chip[:7].lower()}x.cfg"
        if not Path("/usr/share/openocd/scripts", chip_script).exists():
            chip_script = f"target/{self._chip[:7].lower()}.cfg"
        openocd_args = [
            "-f",
            "interface/stlink-v2-1.cfg",
            "-f",
            chip_script,
            "-f",
            ".devcontainer/stm32.cfg",
        ]
        self._debug_embedded(openocd_args, "gdb-multiarch")

    def _debug_esp32(self) -> None:
        openocd_args = [
            "-f",
            "interface/ftdi/esp32_devkitj_v1.cfg",
            "-f",
            "board/esp-wroom-32.cfg",
        ]
        self._debug_embedded(openocd_args, "xtensa-esp32-elf-gdb")

    def _get_bin_path(self, bin_name: str) -> Path:
        if self._target.family == "esp32":
            bin_path = Path(self._project_root, "build/Debug", bin_name)
        else:
            bin_path = Path(self._project_root, "build/Debug/bin", bin_name)
        if self._target.family in ("stm32", "esp32") and bin_path.suffix != "elf":
            bin_path = bin_path.with_suffix(".elf")
        return bin_path


def scargo_debug(bin_path: Optional[Path]) -> None:
    config = prepare_config(run_in_docker=False)
    if config.project.target.family != "x86":
        run_scargo_again_in_docker(config.project, config.project_root)
    debug = _ScargoDebug(config, bin_path)
    debug.run_debugger()



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #
"""Format project code using formatter"""
import os
from typing import List, Type

from scargo.commands.check import (
    CheckerFixer,
    ClangFormatChecker,
    CopyrightChecker,
    PragmaChecker,
)
from scargo.config_utils import prepare_config


def scargo_fix(pragma: bool, copy_right: bool, clang_format: bool) -> None:
    """
    Fix format

    :param bool pragma: fix pragma format
    :param bool copy_right: fix copyrights
    :param bool clang_format: fix clang format
    :return: None
    """
    config = prepare_config()

    checkers: List[Type[CheckerFixer]] = []
    if pragma:
        checkers.append(PragmaChecker)
    if copy_right:
        checkers.append(CopyrightChecker)
    if clang_format:
        checkers.append(ClangFormatChecker)

    if not checkers:
        checkers = [PragmaChecker, CopyrightChecker, ClangFormatChecker]

    # Todo, remove chdir and change cwd for checks
    os.chdir(config.project_root)

    for checker_class in checkers:
        checker_class(config, fix_errors=True).check()



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

import os
import shutil
from pathlib import Path
from typing import Any, Dict

from scargo import __version__
from scargo.config import Config
from scargo.file_generators.base_gen import create_file_from_template
from scargo.global_values import SCARGO_PKG_PATH


class _DockerComposeTemplate:
    """
    This class is a container for docker compose yaml files creation with multilayer approach
    """

    def __init__(self, config: Config, docker_path: Path):
        self.docker_path = docker_path
        self._config = config

    def generate_docker_env(self) -> None:
        """Generate dirs and files"""
        self._create_file_from_template(
            "docker/Dockerfile-custom.j2",
            "Dockerfile-custom",
            template_params={},
            overwrite=False,
        )
        self._create_file_from_template(
            "docker/docker-compose.yaml.j2",
            "docker-compose.yaml",
            template_params={"config": self._config},
        )
        self._create_file_from_template(
            "docker/devcontainer.json.j2",
            "devcontainer.json",
            template_params={"project": self._config.project},
        )
        if self._config.project.target.family == "stm32":
            self._create_file_from_template(
                "docker/stm32.cfg.j2",
                "stm32.cfg",
                template_params={},
            )

        custom_docker = self._get_dockerfile_custom_content()
        scargo_package_version = self._set_up_package_version()

        self._create_file_from_template(
            "docker/Dockerfile.j2",
            "Dockerfile",
            template_params={
                "project": self._config.project,
                "scargo_package_version": scargo_package_version,
                "custom_docker": custom_docker,
            },
        )

    def _create_file_from_template(
        self,
        template_path: str,
        output_filename: str,
        template_params: Dict[str, Any],
        overwrite: bool = True,
    ) -> None:
        create_file_from_template(
            template_path,
            self.docker_path / output_filename,
            template_params=template_params,
            config=self._config,
            overwrite=overwrite,
        )

    def _get_dockerfile_custom_content(self) -> str:
        project_root = self._config.project_root
        custom_docker_path = project_root / self._config.project.docker_file
        if custom_docker_path.is_file():
            return custom_docker_path.read_text()
        return ""

    def _set_up_package_version(self) -> str:
        if whl_path_str := os.getenv("SCARGO_DOCKER_INSTALL_LOCAL"):
            repo_root = SCARGO_PKG_PATH.parent
            whl_path = repo_root / whl_path_str
            shutil.copy(whl_path, self.docker_path)
            return whl_path.name
        return f"scargo=={__version__}"


def generate_docker_compose(docker_path: Path, config: Config) -> None:
    docker_compose_template = _DockerComposeTemplate(config, docker_path)
    docker_compose_template.generate_docker_env()



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Run test from project"""
import subprocess
import sys
from pathlib import Path
from typing import List, Union

from scargo.config import Config
from scargo.config_utils import prepare_config
from scargo.logger import get_logger

logger = get_logger()


def scargo_test(
    verbose: bool, profile: str = "Debug", detailed_coverage: bool = False
) -> None:
    """
    Run test
    :param bool verbose: if verbose
    :param str profile: CMake profile to use
    :param bool detailed_coverage: Generate detailed coverage HTML files
    """
    config = prepare_config()

    test_dir_name = "tests"
    project_dir = config.project_root
    tests_src_dir = project_dir / test_dir_name
    test_build_dir = project_dir / "build" / test_dir_name

    if not tests_src_dir.exists():
        logger.error("Directory `tests` does not exist.")
        sys.exit(1)

    if not Path(tests_src_dir, "CMakeLists.txt").exists():
        logger.error("Directory `tests`: File `CMakeLists.txt` does not exist.")
        logger.info("Did you run `scargo update`?")
        sys.exit(1)

    test_build_dir.mkdir(parents=True, exist_ok=True)

    try:
        # Run CMake and build tests.
        subprocess.check_call(
            [
                "conan",
                "install",
                tests_src_dir,
                "-if",
                test_build_dir,
                f"-sbuild_type={profile}",
            ],
            cwd=project_dir,
        )
        subprocess.check_call(
            ["conan", "build", tests_src_dir, "-bf", test_build_dir],
            cwd=project_dir,
        )
    except subprocess.CalledProcessError:
        logger.error("Failed to build tests.")
        sys.exit(1)

    # run ut
    run_ut(config, verbose, test_build_dir, detailed_coverage)


def run_ut(config: Config, verbose: bool, cwd: Path, detailed_coverage: bool) -> None:
    # Run tests.
    cmd: List[Union[str, Path]] = ["ctest"]

    if verbose:
        cmd.append("--verbose")
    try:
        # Using `subprocess.run` because tests can fail,
        # and we do not want Python to throw exception.
        subprocess.run(cmd, cwd=cwd, check=False)

        output_option = (
            "--html-details=ut-coverage.details.html"
            if detailed_coverage
            else "--html=ut-coverage.html"
        )

        # Run code coverage.
        cmd = [
            "gcovr",
            "-r",
            "ut",
            ".",
            "-f",
            config.source_dir_path,
            output_option,
        ]
        print(cmd)

        gcov_executable = config.tests.gcov_executable

        if gcov_executable != "":
            cmd.extend(["--gcov-executable", gcov_executable])

        subprocess.check_call(cmd, cwd=cwd)
    except subprocess.CalledProcessError:
        logger.error("Fail to run project tests")
        sys.exit(1)


def run_it(config: Config, verbose: bool) -> None:
    # Run tests.
    cmd = ["ctest"]

    if verbose:
        cmd.append("--verbose")

    subprocess.run(cmd, check=False)
    # Using `subprocess.run` because tests
    # can fail, and we do not want Python to
    # throw exception.

    # Run code coverage.
    cmd = [
        "gcovr",
        "-r",
        "it",
        ".",
        "--txt",
        "it-coverage.txt",
        "--html=it-coverage.html",
    ]

    gcov_executable = config.tests.gcov_executable

    if gcov_executable != "":
        cmd.extend(["--gcov-executable", gcov_executable])

    subprocess.check_call(cmd)
    subprocess.check_call("cat it-coverage.txt", shell=True)



# #
# @copyright Copyright (C) 2023 SpyroSoft Solutions S.A. All rights reserved.
# #

"""Create new project"""
import re
import subprocess
import sys
from pathlib import Path
from typing import Optional

from scargo import __version__
from scargo.config import Target
from scargo.config_utils import get_scargo_config_or_exit
from scargo.file_generators.cpp_gen import generate_cpp
from scargo.file_generators.toml_gen import generate_toml
from scargo.global_values import SCARGO_DEFAULT_CONFIG_FILE, SCARGO_DOCKER_ENV
from scargo.logger import get_logger

logger = get_logger()


def scargo_new(
    name: str,
    bin_name: Optional[str],
    lib_name: Optional[str],
    target: Target,
    create_docker: bool,
    git: bool,
) -> None:
    """
    Create new project

    :param str name: name of project
    :param str bin_name: name of bin file
    :param str lib_name: name of lib file
    :param Target target: architecture type
    :param bool create_docker: initialize docker environment
    :param bool git: initialize git repository
    :return: None
    :raises FileExistsError: if project with provided name exist
    """
    if not re.match(r"[a-zA-Z][\w-]*$", name):
        logger.error(
            "Name must consist of letters, digits, dash and undescore only,"
            " and the first character must be a letter"
        )
        sys.exit(1)

    # If neither binary target nor library target is specified then create a
    # binary target named same as the project name.
    if not bin_name and not lib_name:
        bin_name = name  # One item tuple.

    project_dir = Path(name)
    try:
        project_dir.mkdir()
    except FileExistsError:
        logger.error("Provided project name: %s already exist.", name)
        sys.exit(1)

    build_env = get_build_env(create_docker)

    toml_path = project_dir / SCARGO_DEFAULT_CONFIG_FILE
    generate_toml(
        toml_path,
        project_name=name,
        target=target,
        build_env=build_env,
        cflags="-Wall -Wextra",
        cxxflags="-Wall -Wextra",
        version=__version__,
        docker_image_tag=f"{name.lower()}-dev:1.0",
        lib_name=lib_name,
        bin_name=bin_name,
    )

    config = get_scargo_config_or_exit(toml_path)
    generate_cpp(config)

    test_dir = project_dir / "tests"
    Path(test_dir, "mocks").mkdir(parents=True)
    Path(test_dir, "ut").mkdir(parents=True)
    Path(test_dir, "it").mkdir(parents=True)

    if git:
        subprocess.check_call("git init -q", shell=True, cwd=project_dir)
        logger.info("Initialized git repo")


def get_build_env(create_docker: bool) -> str:
    """
    Get build env
    :param bool create_docker: if create docker
    :return: build env
    """
    if create_docker:
        build_env = f"{SCARGO_DOCKER_ENV}"
    else:
        build_env = "native"
    return build_env
